import { Mapper } from '@mobicoop/ddd-library';
import { AdResponseDto } from './interface/dtos/ad.response.dto';
import { Inject, Injectable } from '@nestjs/common';
import { AdEntity } from './core/domain/ad.entity';
import {
  AdWriteModel,
  AdReadModel,
  WaypointModel,
  ScheduleItemModel,
} from './infrastructure/ad.repository';
import { Frequency } from './core/domain/ad.types';
import { WaypointProps } from './core/domain/value-objects/waypoint.value-object';
import { v4 } from 'uuid';
import { ScheduleItemProps } from './core/domain/value-objects/schedule-item.value-object';
import { OUTPUT_DATETIME_TRANSFORMER } from './ad.di-tokens';
import { DateTimeTransformerPort } from './core/application/ports/datetime-transformer.port';

/**
 * Mapper constructs objects that are used in different layers:
 * Record is an object that is stored in a database,
 * Entity is an object that is used in application domain layer,
 * and a ResponseDTO is an object returned to a user (usually as json).
 */

@Injectable()
export class AdMapper
  implements Mapper<AdEntity, AdReadModel, AdWriteModel, AdResponseDto>
{
  constructor(
    @Inject(OUTPUT_DATETIME_TRANSFORMER)
    private readonly outputDatetimeTransformer: DateTimeTransformerPort,
  ) {}

  toPersistence = (entity: AdEntity): AdWriteModel => {
    const copy = entity.getProps();
    const now = new Date();
    const record: AdWriteModel = {
      uuid: copy.id,
      userUuid: copy.userId,
      driver: copy.driver,
      passenger: copy.passenger,
      frequency: copy.frequency,
      fromDate: new Date(copy.fromDate),
      toDate: new Date(copy.toDate),
      schedule: {
        create: copy.schedule.map((scheduleItem: ScheduleItemProps) => ({
          uuid: v4(),
          day: scheduleItem.day,
          time: new Date(
            1970,
            0,
            1,
            parseInt(scheduleItem.time.split(':')[0]),
            parseInt(scheduleItem.time.split(':')[1]),
          ),
          margin: scheduleItem.margin,
          createdAt: now,
          updatedAt: now,
        })),
      },
      seatsProposed: copy.seatsProposed,
      seatsRequested: copy.seatsRequested,
      strict: copy.strict,
      waypoints: {
        create: copy.waypoints.map((waypoint: WaypointProps) => ({
          uuid: v4(),
          position: waypoint.position,
          name: waypoint.address.name,
          houseNumber: waypoint.address.houseNumber,
          street: waypoint.address.street,
          locality: waypoint.address.locality,
          postalCode: waypoint.address.postalCode,
          country: waypoint.address.country,
          lon: waypoint.address.coordinates.lon,
          lat: waypoint.address.coordinates.lat,
          createdAt: now,
          updatedAt: now,
        })),
      },
      createdAt: copy.createdAt,
      updatedAt: copy.updatedAt,
    };
    return record;
  };

  toDomain = (record: AdReadModel): AdEntity => {
    const entity = new AdEntity({
      id: record.uuid,
      createdAt: new Date(record.createdAt),
      updatedAt: new Date(record.updatedAt),
      props: {
        userId: record.userUuid,
        driver: record.driver,
        passenger: record.passenger,
        frequency: Frequency[record.frequency],
        fromDate: record.fromDate.toISOString().split('T')[0],
        toDate: record.toDate.toISOString().split('T')[0],
        schedule: record.schedule.map((scheduleItem: ScheduleItemModel) => ({
          day: scheduleItem.day,
          time: `${scheduleItem.time
            .getUTCHours()
            .toString()
            .padStart(2, '0')}:${scheduleItem.time
            .getUTCMinutes()
            .toString()
            .padStart(2, '0')}`,
          margin: scheduleItem.margin,
        })),
        seatsProposed: record.seatsProposed,
        seatsRequested: record.seatsRequested,
        strict: record.strict,
        waypoints: record.waypoints.map((waypoint: WaypointModel) => ({
          position: waypoint.position,
          address: {
            name: waypoint.name,
            houseNumber: waypoint.houseNumber,
            street: waypoint.street,
            postalCode: waypoint.postalCode,
            locality: waypoint.locality,
            country: waypoint.country,
            coordinates: {
              lon: waypoint.lon,
              lat: waypoint.lat,
            },
          },
        })),
      },
    });
    return entity;
  };

  toResponse = (entity: AdEntity): AdResponseDto => {
    const props = entity.getProps();
    const response = new AdResponseDto(entity);
    response.userId = props.userId;
    response.driver = props.driver;
    response.passenger = props.passenger;
    response.frequency = props.frequency;
    response.fromDate = this.outputDatetimeTransformer.fromDate(
      {
        date: props.fromDate,
        time: props.schedule[0].time,
        coordinates: props.waypoints[0].address.coordinates,
      },
      props.frequency,
    );
    response.toDate = this.outputDatetimeTransformer.toDate(
      props.toDate,
      {
        date: props.fromDate,
        time: props.schedule[0].time,
        coordinates: props.waypoints[0].address.coordinates,
      },
      props.frequency,
    );
    response.schedule = props.schedule.map(
      (scheduleItem: ScheduleItemProps) => ({
        day: this.outputDatetimeTransformer.day(
          scheduleItem.day,
          {
            date: props.fromDate,
            time: scheduleItem.time,
            coordinates: props.waypoints[0].address.coordinates,
          },
          props.frequency,
        ),
        time: this.outputDatetimeTransformer.time(
          {
            date: props.fromDate,
            time: scheduleItem.time,
            coordinates: props.waypoints[0].address.coordinates,
          },
          props.frequency,
        ),
        margin: scheduleItem.margin,
      }),
    );
    response.seatsProposed = props.seatsProposed;
    response.seatsRequested = props.seatsRequested;
    response.waypoints = props.waypoints.map((waypoint: WaypointProps) => ({
      position: waypoint.position,
      name: waypoint.address.name,
      houseNumber: waypoint.address.houseNumber,
      street: waypoint.address.street,
      postalCode: waypoint.address.postalCode,
      locality: waypoint.address.locality,
      country: waypoint.address.country,
      lon: waypoint.address.coordinates.lon,
      lat: waypoint.address.coordinates.lat,
    }));
    return response;
  };
}
