import { AdEntity } from '@modules/ad/core/domain/ad.entity';
import {
  CreateAdProps,
  DefaultAdProps,
  Frequency,
} from '@modules/ad/core/domain/ad.types';
import { WaypointProps } from '@modules/ad/core/domain/value-objects/waypoint.value-object';

const originWaypointProps: WaypointProps = {
  position: 0,
  address: {
    houseNumber: '5',
    street: 'Avenue Foch',
    locality: 'Nancy',
    postalCode: '54000',
    country: 'France',
    coordinates: {
      lat: 48.689445,
      lon: 6.17651,
    },
  },
};
const destinationWaypointProps: WaypointProps = {
  position: 1,
  address: {
    locality: 'Paris',
    postalCode: '75000',
    country: 'France',
    coordinates: {
      lat: 48.8566,
      lon: 2.3522,
    },
  },
};

const baseCreateAdProps = {
  userId: 'e8fe64b1-4c33-49e1-9f69-4db48b21df36',
  seatsProposed: 3,
  seatsRequested: 1,
  strict: false,
  waypoints: [originWaypointProps, destinationWaypointProps],
};
const punctualCreateAdProps = {
  fromDate: '2023-06-21',
  toDate: '2023-06-21',
  schedule: [
    {
      day: 3,
      time: '08:30',
    },
  ],
  frequency: Frequency.PUNCTUAL,
};
const recurrentCreateAdProps = {
  fromDate: '2023-06-21',
  toDate: '2024-06-20',
  schedule: [
    {
      day: 1,
      time: '08:30',
      margin: 600,
    },
    {
      day: 2,
      time: '08:30',
      margin: 600,
    },
    {
      day: 3,
      time: '08:00',
      margin: 600,
    },
    {
      day: 4,
      time: '08:30',
      margin: 600,
    },
    {
      day: 5,
      time: '08:30',
      margin: 600,
    },
  ],
  frequency: Frequency.RECURRENT,
};
const punctualPassengerCreateAdProps: CreateAdProps = {
  ...baseCreateAdProps,
  ...punctualCreateAdProps,
  driver: false,
  passenger: true,
};
const recurrentPassengerCreateAdProps: CreateAdProps = {
  ...baseCreateAdProps,
  ...recurrentCreateAdProps,
  driver: false,
  passenger: true,
};
const punctualDriverCreateAdProps: CreateAdProps = {
  ...baseCreateAdProps,
  ...punctualCreateAdProps,
  driver: true,
  passenger: false,
};
const recurrentDriverCreateAdProps: CreateAdProps = {
  ...baseCreateAdProps,
  ...recurrentCreateAdProps,
  driver: true,
  passenger: false,
};
const punctualDriverPassengerCreateAdProps: CreateAdProps = {
  ...baseCreateAdProps,
  ...punctualCreateAdProps,
  driver: true,
  passenger: true,
};
const recurrentDriverPassengerCreateAdProps: CreateAdProps = {
  ...baseCreateAdProps,
  ...recurrentCreateAdProps,
  driver: true,
  passenger: true,
};
const defaultAdProps: DefaultAdProps = {
  driver: false,
  passenger: true,
  marginDuration: 900,
  seatsProposed: 3,
  seatsRequested: 1,
  strict: false,
};

describe('Ad entity create', () => {
  describe('With complete props', () => {
    it('should create a new punctual passenger ad entity', async () => {
      const punctualPassengerAd: AdEntity = AdEntity.create(
        punctualPassengerCreateAdProps,
        defaultAdProps,
      );
      expect(punctualPassengerAd.id.length).toBe(36);
      expect(punctualPassengerAd.getProps().schedule.length).toBe(1);
      expect(punctualPassengerAd.getProps().schedule[0].day).toBe(3);
      expect(punctualPassengerAd.getProps().schedule[0].time).toBe('08:30');
      expect(punctualPassengerAd.getProps().driver).toBeFalsy();
      expect(punctualPassengerAd.getProps().passenger).toBeTruthy();
    });
    it('should create a new punctual driver ad entity', async () => {
      const punctualDriverAd: AdEntity = AdEntity.create(
        punctualDriverCreateAdProps,
        defaultAdProps,
      );
      expect(punctualDriverAd.id.length).toBe(36);
      expect(punctualDriverAd.getProps().schedule.length).toBe(1);
      expect(punctualDriverAd.getProps().schedule[0].day).toBe(3);
      expect(punctualDriverAd.getProps().schedule[0].time).toBe('08:30');
      expect(punctualDriverAd.getProps().driver).toBeTruthy();
      expect(punctualDriverAd.getProps().passenger).toBeFalsy();
    });
    it('should create a new punctual driver and passenger ad entity', async () => {
      const punctualDriverPassengerAd: AdEntity = AdEntity.create(
        punctualDriverPassengerCreateAdProps,
        defaultAdProps,
      );
      expect(punctualDriverPassengerAd.id.length).toBe(36);
      expect(punctualDriverPassengerAd.getProps().schedule.length).toBe(1);
      expect(punctualDriverPassengerAd.getProps().schedule[0].day).toBe(3);
      expect(punctualDriverPassengerAd.getProps().schedule[0].time).toBe(
        '08:30',
      );
      expect(punctualDriverPassengerAd.getProps().driver).toBeTruthy();
      expect(punctualDriverPassengerAd.getProps().passenger).toBeTruthy();
    });
    it('should create a new recurrent passenger ad entity', async () => {
      const recurrentPassengerAd: AdEntity = AdEntity.create(
        recurrentPassengerCreateAdProps,
        defaultAdProps,
      );
      expect(recurrentPassengerAd.id.length).toBe(36);
      expect(recurrentPassengerAd.getProps().schedule.length).toBe(5);
      expect(recurrentPassengerAd.getProps().schedule[0].day).toBe(1);
      expect(recurrentPassengerAd.getProps().schedule[2].time).toBe('08:00');
      expect(recurrentPassengerAd.getProps().driver).toBeFalsy();
      expect(recurrentPassengerAd.getProps().passenger).toBeTruthy();
    });
    it('should create a new recurrent driver ad entity', async () => {
      const recurrentDriverAd: AdEntity = AdEntity.create(
        recurrentDriverCreateAdProps,
        defaultAdProps,
      );
      expect(recurrentDriverAd.id.length).toBe(36);
      expect(recurrentDriverAd.getProps().schedule.length).toBe(5);
      expect(recurrentDriverAd.getProps().schedule[1].day).toBe(2);
      expect(recurrentDriverAd.getProps().schedule[0].time).toBe('08:30');
      expect(recurrentDriverAd.getProps().driver).toBeTruthy();
      expect(recurrentDriverAd.getProps().passenger).toBeFalsy();
    });
    it('should create a new recurrent driver and passenger ad entity', async () => {
      const recurrentDriverPassengerAd: AdEntity = AdEntity.create(
        recurrentDriverPassengerCreateAdProps,
        defaultAdProps,
      );
      expect(recurrentDriverPassengerAd.id.length).toBe(36);
      expect(recurrentDriverPassengerAd.getProps().schedule.length).toBe(5);
      expect(recurrentDriverPassengerAd.getProps().schedule[3].day).toBe(4);
      expect(recurrentDriverPassengerAd.getProps().schedule[4].time).toBe(
        '08:30',
      );
      expect(recurrentDriverPassengerAd.getProps().driver).toBeTruthy();
      expect(recurrentDriverPassengerAd.getProps().passenger).toBeTruthy();
    });
  });

  describe('With incomplete props', () => {
    it('should create a new punctual passenger ad entity if no role is given', async () => {
      const punctualWithoutRoleCreateAdProps: CreateAdProps = {
        ...baseCreateAdProps,
        ...punctualCreateAdProps,
        driver: false,
        passenger: false,
      };
      const punctualWithoutRoleAd: AdEntity = AdEntity.create(
        punctualWithoutRoleCreateAdProps,
        defaultAdProps,
      );
      expect(punctualWithoutRoleAd.id.length).toBe(36);
      expect(punctualWithoutRoleAd.getProps().schedule.length).toBe(1);
      expect(punctualWithoutRoleAd.getProps().schedule[0].time).toBe('08:30');
      expect(punctualWithoutRoleAd.getProps().driver).toBeFalsy();
      expect(punctualWithoutRoleAd.getProps().passenger).toBeTruthy();
    });
    it('should create a new strict punctual passenger ad entity if no strict param is given', async () => {
      const punctualWithoutStrictCreateAdProps: CreateAdProps = {
        userId: 'e8fe64b1-4c33-49e1-9f69-4db48b21df36',
        seatsProposed: 3,
        seatsRequested: 1,
        strict: undefined,
        waypoints: [originWaypointProps, destinationWaypointProps],
        ...punctualCreateAdProps,
        driver: false,
        passenger: true,
      };
      const punctualWithoutStrictAd: AdEntity = AdEntity.create(
        punctualWithoutStrictCreateAdProps,
        defaultAdProps,
      );
      expect(punctualWithoutStrictAd.id.length).toBe(36);
      expect(punctualWithoutStrictAd.getProps().schedule.length).toBe(1);
      expect(punctualWithoutStrictAd.getProps().schedule[0].time).toBe('08:30');
      expect(punctualWithoutStrictAd.getProps().driver).toBeFalsy();
      expect(punctualWithoutStrictAd.getProps().passenger).toBeTruthy();
      expect(punctualWithoutStrictAd.getProps().strict).toBeFalsy();
    });
    it('should create a new punctual passenger ad entity with seats requested if no seats requested param is given', async () => {
      const punctualWithoutSeatsRequestedCreateAdProps: CreateAdProps = {
        userId: 'e8fe64b1-4c33-49e1-9f69-4db48b21df36',
        seatsProposed: 3,
        seatsRequested: undefined,
        strict: false,
        waypoints: [originWaypointProps, destinationWaypointProps],
        ...punctualCreateAdProps,
        driver: false,
        passenger: true,
      };
      const punctualWithoutSeatsRequestedAd: AdEntity = AdEntity.create(
        punctualWithoutSeatsRequestedCreateAdProps,
        defaultAdProps,
      );
      expect(punctualWithoutSeatsRequestedAd.id.length).toBe(36);
      expect(punctualWithoutSeatsRequestedAd.getProps().schedule.length).toBe(
        1,
      );
      expect(punctualWithoutSeatsRequestedAd.getProps().schedule[0].time).toBe(
        '08:30',
      );
      expect(punctualWithoutSeatsRequestedAd.getProps().driver).toBeFalsy();
      expect(punctualWithoutSeatsRequestedAd.getProps().passenger).toBeTruthy();
      expect(punctualWithoutSeatsRequestedAd.getProps().seatsRequested).toBe(1);
    });
    it('should create a new punctual driver ad entity with seats proposed if no seats proposed param is given', async () => {
      const punctualWithoutSeatsProposedCreateAdProps: CreateAdProps = {
        userId: 'e8fe64b1-4c33-49e1-9f69-4db48b21df36',
        seatsProposed: undefined,
        seatsRequested: 1,
        strict: false,
        waypoints: [originWaypointProps, destinationWaypointProps],
        ...punctualCreateAdProps,
        driver: true,
        passenger: false,
      };
      const punctualWithoutSeatsProposedAd: AdEntity = AdEntity.create(
        punctualWithoutSeatsProposedCreateAdProps,
        defaultAdProps,
      );
      expect(punctualWithoutSeatsProposedAd.id.length).toBe(36);
      expect(punctualWithoutSeatsProposedAd.getProps().schedule.length).toBe(1);
      expect(punctualWithoutSeatsProposedAd.getProps().schedule[0].time).toBe(
        '08:30',
      );
      expect(punctualWithoutSeatsProposedAd.getProps().driver).toBeTruthy();
      expect(punctualWithoutSeatsProposedAd.getProps().passenger).toBeFalsy();
      expect(punctualWithoutSeatsProposedAd.getProps().seatsProposed).toBe(3);
    });
    it('should create a new punctual driver ad entity with margin durations if margin durations are empty', async () => {
      const punctualWithoutMarginDurationCreateAdProps: CreateAdProps = {
        ...baseCreateAdProps,
        waypoints: [originWaypointProps, destinationWaypointProps],
        ...punctualCreateAdProps,
        driver: true,
        passenger: false,
      };
      const punctualWithoutMarginDurationAd: AdEntity = AdEntity.create(
        punctualWithoutMarginDurationCreateAdProps,
        defaultAdProps,
      );
      expect(punctualWithoutMarginDurationAd.id.length).toBe(36);
      expect(punctualWithoutMarginDurationAd.getProps().schedule.length).toBe(
        1,
      );
      expect(punctualWithoutMarginDurationAd.getProps().schedule[0].time).toBe(
        '08:30',
      );
      expect(
        punctualWithoutMarginDurationAd.getProps().schedule[0].margin,
      ).toBe(900);
      expect(punctualWithoutMarginDurationAd.getProps().driver).toBeTruthy();
      expect(punctualWithoutMarginDurationAd.getProps().passenger).toBeFalsy();
    });
    it('should create a new punctual passenger ad entity with valid positions if positions are missing', async () => {
      const punctualWithoutPositionsCreateAdProps: CreateAdProps = {
        userId: 'e8fe64b1-4c33-49e1-9f69-4db48b21df36',
        seatsProposed: 3,
        seatsRequested: 1,
        strict: undefined,
        waypoints: [
          {
            position: undefined,
            address: {
              houseNumber: '5',
              street: 'Avenue Foch',
              locality: 'Nancy',
              postalCode: '54000',
              country: 'France',
              coordinates: {
                lat: 48.689445,
                lon: 6.17651,
              },
            },
          },
          {
            position: undefined,
            address: {
              locality: 'Paris',
              postalCode: '75000',
              country: 'France',
              coordinates: {
                lat: 48.8566,
                lon: 2.3522,
              },
            },
          },
        ],
        ...punctualCreateAdProps,
        driver: false,
        passenger: false,
      };
      const punctualWithoutPositionsAd: AdEntity = AdEntity.create(
        punctualWithoutPositionsCreateAdProps,
        defaultAdProps,
      );
      expect(punctualWithoutPositionsAd.id.length).toBe(36);
      expect(punctualWithoutPositionsAd.getProps().schedule.length).toBe(1);
      expect(punctualWithoutPositionsAd.getProps().schedule[0].time).toBe(
        '08:30',
      );
      expect(punctualWithoutPositionsAd.getProps().driver).toBeFalsy();
      expect(punctualWithoutPositionsAd.getProps().passenger).toBeTruthy();
      expect(punctualWithoutPositionsAd.getProps().waypoints[0].position).toBe(
        0,
      );
      expect(punctualWithoutPositionsAd.getProps().waypoints[1].position).toBe(
        1,
      );
    });
  });
});
