import { TimeConverter } from '@modules/ad/infrastructure/time-converter';

describe('Time Converter', () => {
  it('should be defined', () => {
    const timeConverter: TimeConverter = new TimeConverter();
    expect(timeConverter).toBeDefined();
  });

  describe('localStringTimeToUtcStringTime', () => {
    it('should convert a paris time to utc time', () => {
      const timeConverter: TimeConverter = new TimeConverter();
      const parisTime = '08:00';
      const utcDatetime = timeConverter.localStringTimeToUtcStringTime(
        parisTime,
        'Europe/Paris',
      );
      expect(utcDatetime).toBe('07:00');
    });
    it('should return undefined if time is invalid', () => {
      const timeConverter: TimeConverter = new TimeConverter();
      const parisTime = '28:00';
      const utcDatetime = timeConverter.localStringTimeToUtcStringTime(
        parisTime,
        'Europe/Paris',
      );
      expect(utcDatetime).toBeUndefined();
    });
    it('should return undefined if time is undefined', () => {
      const timeConverter: TimeConverter = new TimeConverter();
      const parisTime = undefined;
      const utcDatetime = timeConverter.localStringTimeToUtcStringTime(
        parisTime,
        'Europe/Paris',
      );
      expect(utcDatetime).toBeUndefined();
    });
    it('should return undefined if timezone is invalid', () => {
      const timeConverter: TimeConverter = new TimeConverter();
      const fooBarTime = '08:00';
      const utcDatetime = timeConverter.localStringTimeToUtcStringTime(
        fooBarTime,
        'Foo/Bar',
      );
      expect(utcDatetime).toBeUndefined();
    });
    it('should return undefined if timezone is undefined', () => {
      const timeConverter: TimeConverter = new TimeConverter();
      const fooBarTime = '08:00';
      const utcDatetime = timeConverter.localStringTimeToUtcStringTime(
        fooBarTime,
        undefined,
      );
      expect(utcDatetime).toBeUndefined();
    });
  });

  describe('utcStringTimeToLocalStringTime', () => {
    it('should convert a utc time to a paris time', () => {
      const timeConverter: TimeConverter = new TimeConverter();
      const utcTime = '07:00';
      const parisTime = timeConverter.utcStringTimeToLocalStringTime(
        utcTime,
        'Europe/Paris',
      );
      expect(parisTime).toBe('08:00');
    });
    it('should return undefined if time is invalid', () => {
      const timeConverter: TimeConverter = new TimeConverter();
      const utcTime = '27:00';
      const parisTime = timeConverter.utcStringTimeToLocalStringTime(
        utcTime,
        'Europe/Paris',
      );
      expect(parisTime).toBeUndefined();
    });
    it('should return undefined if time is undefined', () => {
      const timeConverter: TimeConverter = new TimeConverter();
      const utcTime = undefined;
      const parisTime = timeConverter.utcStringTimeToLocalStringTime(
        utcTime,
        'Europe/Paris',
      );
      expect(parisTime).toBeUndefined();
    });
    it('should return undefined if timezone is invalid', () => {
      const timeConverter: TimeConverter = new TimeConverter();
      const utcTime = '07:00';
      const parisTime = timeConverter.utcStringTimeToLocalStringTime(
        utcTime,
        'Foo/Bar',
      );
      expect(parisTime).toBeUndefined();
    });
    it('should return undefined if timezone is undefined', () => {
      const timeConverter: TimeConverter = new TimeConverter();
      const utcTime = '07:00';
      const parisTime = timeConverter.utcStringTimeToLocalStringTime(
        utcTime,
        undefined,
      );
      expect(parisTime).toBeUndefined();
    });
  });

  describe('localStringDateTimeToUtcDate', () => {
    it('should convert a summer paris date and time to a utc date', () => {
      const timeConverter: TimeConverter = new TimeConverter();
      const parisDate = '2023-06-22';
      const parisTime = '12:00';
      const utcDate = timeConverter.localStringDateTimeToUtcDate(
        parisDate,
        parisTime,
        'Europe/Paris',
      );
      expect(utcDate.toISOString()).toBe('2023-06-22T10:00:00.000Z');
    });
    it('should convert a winter paris date and time to a utc date', () => {
      const timeConverter: TimeConverter = new TimeConverter();
      const parisDate = '2023-02-02';
      const parisTime = '12:00';
      const utcDate = timeConverter.localStringDateTimeToUtcDate(
        parisDate,
        parisTime,
        'Europe/Paris',
      );
      expect(utcDate.toISOString()).toBe('2023-02-02T11:00:00.000Z');
    });
    it('should convert a summer paris date and time to a utc date without dst', () => {
      const timeConverter: TimeConverter = new TimeConverter();
      const parisDate = '2023-06-22';
      const parisTime = '12:00';
      const utcDate = timeConverter.localStringDateTimeToUtcDate(
        parisDate,
        parisTime,
        'Europe/Paris',
        false,
      );
      expect(utcDate.toISOString()).toBe('2023-06-22T11:00:00.000Z');
    });
    it('should convert a tonga date and time to a utc date', () => {
      const timeConverter: TimeConverter = new TimeConverter();
      const tongaDate = '2023-02-02';
      const tongaTime = '12:00';
      const utcDate = timeConverter.localStringDateTimeToUtcDate(
        tongaDate,
        tongaTime,
        'Pacific/Tongatapu',
      );
      expect(utcDate.toISOString()).toBe('2023-02-01T23:00:00.000Z');
    });
    it('should convert a papeete date and time to a utc date', () => {
      const timeConverter: TimeConverter = new TimeConverter();
      const papeeteDate = '2023-02-02';
      const papeeteTime = '15:00';
      const utcDate = timeConverter.localStringDateTimeToUtcDate(
        papeeteDate,
        papeeteTime,
        'Pacific/Tahiti',
      );
      expect(utcDate.toISOString()).toBe('2023-02-03T01:00:00.000Z');
    });
    it('should return undefined if date is invalid', () => {
      const timeConverter: TimeConverter = new TimeConverter();
      const parisDate = '2023-06-32';
      const parisTime = '08:00';
      const utcDate = timeConverter.localStringDateTimeToUtcDate(
        parisDate,
        parisTime,
        'Europe/Paris',
      );
      expect(utcDate).toBeUndefined();
    });
    it('should return undefined if date is undefined', () => {
      const timeConverter: TimeConverter = new TimeConverter();
      const parisDate = undefined;
      const parisTime = '08:00';
      const utcDate = timeConverter.localStringDateTimeToUtcDate(
        parisDate,
        parisTime,
        'Europe/Paris',
      );
      expect(utcDate).toBeUndefined();
    });
    it('should return undefined if time is invalid', () => {
      const timeConverter: TimeConverter = new TimeConverter();
      const parisDate = '2023-06-22';
      const parisTime = '28:00';
      const utcDate = timeConverter.localStringDateTimeToUtcDate(
        parisDate,
        parisTime,
        'Europe/Paris',
      );
      expect(utcDate).toBeUndefined();
    });
    it('should return undefined if time is undefined', () => {
      const timeConverter: TimeConverter = new TimeConverter();
      const parisDate = '2023-06-22';
      const parisTime = undefined;
      const utcDate = timeConverter.localStringDateTimeToUtcDate(
        parisDate,
        parisTime,
        'Europe/Paris',
      );
      expect(utcDate).toBeUndefined();
    });
    it('should return undefined if timezone is invalid', () => {
      const timeConverter: TimeConverter = new TimeConverter();
      const parisDate = '2023-06-22';
      const parisTime = '12:00';
      const utcDate = timeConverter.localStringDateTimeToUtcDate(
        parisDate,
        parisTime,
        'Foo/Bar',
      );
      expect(utcDate).toBeUndefined();
    });
    it('should return undefined if timezone is undefined', () => {
      const timeConverter: TimeConverter = new TimeConverter();
      const parisDate = '2023-06-22';
      const parisTime = '12:00';
      const utcDate = timeConverter.localStringDateTimeToUtcDate(
        parisDate,
        parisTime,
        undefined,
      );
      expect(utcDate).toBeUndefined();
    });
  });

  describe('utcStringDateTimeToLocalIsoString', () => {
    it('should convert a utc string date and time to a summer paris date isostring', () => {
      const timeConverter: TimeConverter = new TimeConverter();
      const utcDate = '2023-06-22';
      const utcTime = '10:00';
      const localIsoString = timeConverter.utcStringDateTimeToLocalIsoString(
        utcDate,
        utcTime,
        'Europe/Paris',
      );
      expect(localIsoString).toBe('2023-06-22T12:00:00.000+02:00');
    });
    it('should convert a utc string date and time to a winter paris date isostring', () => {
      const timeConverter: TimeConverter = new TimeConverter();
      const utcDate = '2023-02-02';
      const utcTime = '10:00';
      const localIsoString = timeConverter.utcStringDateTimeToLocalIsoString(
        utcDate,
        utcTime,
        'Europe/Paris',
      );
      expect(localIsoString).toBe('2023-02-02T11:00:00.000+01:00');
    });
    it('should convert a utc string date and time to a summer paris date isostring without dst', () => {
      const timeConverter: TimeConverter = new TimeConverter();
      const utcDate = '2023-06-22';
      const utcTime = '10:00';
      const localIsoString = timeConverter.utcStringDateTimeToLocalIsoString(
        utcDate,
        utcTime,
        'Europe/Paris',
        false,
      );
      expect(localIsoString).toBe('2023-06-22T11:00:00.000+01:00');
    });
    it('should convert a utc date to a tonga date isostring', () => {
      const timeConverter: TimeConverter = new TimeConverter();
      const utcDate = '2023-02-01';
      const utcTime = '23:00';
      const localIsoString = timeConverter.utcStringDateTimeToLocalIsoString(
        utcDate,
        utcTime,
        'Pacific/Tongatapu',
      );
      expect(localIsoString).toBe('2023-02-02T12:00:00.000+13:00');
    });
    it('should convert a utc date to a papeete date isostring', () => {
      const timeConverter: TimeConverter = new TimeConverter();
      const utcDate = '2023-02-03';
      const utcTime = '01:00';
      const localIsoString = timeConverter.utcStringDateTimeToLocalIsoString(
        utcDate,
        utcTime,
        'Pacific/Tahiti',
      );
      expect(localIsoString).toBe('2023-02-02T15:00:00.000-10:00');
    });
    it('should return undefined if date is invalid', () => {
      const timeConverter: TimeConverter = new TimeConverter();
      const utcDate = '2023-06-32';
      const utcTime = '07:00';
      const parisTime = timeConverter.utcStringDateTimeToLocalIsoString(
        utcDate,
        utcTime,
        'Europe/Paris',
      );
      expect(parisTime).toBeUndefined();
    });
    it('should return undefined if date is undefined', () => {
      const timeConverter: TimeConverter = new TimeConverter();
      const utcDate = undefined;
      const utcTime = '07:00';
      const parisTime = timeConverter.utcStringDateTimeToLocalIsoString(
        utcDate,
        utcTime,
        'Europe/Paris',
      );
      expect(parisTime).toBeUndefined();
    });
    it('should return undefined if time is invalid', () => {
      const timeConverter: TimeConverter = new TimeConverter();
      const utcDate = '2023-06-22';
      const utcTime = '27:00';
      const parisTime = timeConverter.utcStringDateTimeToLocalIsoString(
        utcDate,
        utcTime,
        'Europe/Paris',
      );
      expect(parisTime).toBeUndefined();
    });
    it('should return undefined if time is undefined', () => {
      const timeConverter: TimeConverter = new TimeConverter();
      const utcDate = '2023-06-22';
      const utcTime = undefined;
      const parisTime = timeConverter.utcStringDateTimeToLocalIsoString(
        utcDate,
        utcTime,
        'Europe/Paris',
      );
      expect(parisTime).toBeUndefined();
    });
    it('should return undefined if timezone is invalid', () => {
      const timeConverter: TimeConverter = new TimeConverter();
      const utcDate = '2023-06-22';
      const utcTime = '07:00';
      const parisTime = timeConverter.utcStringDateTimeToLocalIsoString(
        utcDate,
        utcTime,
        'Foo/Bar',
      );
      expect(parisTime).toBeUndefined();
    });
    it('should return undefined if timezone is undefined', () => {
      const timeConverter: TimeConverter = new TimeConverter();
      const utcDate = '2023-06-22';
      const utcTime = '07:00';
      const parisTime = timeConverter.utcStringDateTimeToLocalIsoString(
        utcDate,
        utcTime,
        undefined,
      );
      expect(parisTime).toBeUndefined();
    });
  });

  describe('utcUnixEpochDayFromTime', () => {
    it('should get the utc day of paris at 12:00', () => {
      const timeConverter: TimeConverter = new TimeConverter();
      expect(
        timeConverter.utcUnixEpochDayFromTime('12:00', 'Europe/Paris'),
      ).toBe(4);
    });
    it('should get the utc day of paris at 00:00', () => {
      const timeConverter: TimeConverter = new TimeConverter();
      expect(
        timeConverter.utcUnixEpochDayFromTime('00:00', 'Europe/Paris'),
      ).toBe(3);
    });
    it('should get the utc day of papeete at 16:00', () => {
      const timeConverter: TimeConverter = new TimeConverter();
      expect(
        timeConverter.utcUnixEpochDayFromTime('16:00', 'Pacific/Tahiti'),
      ).toBe(5);
    });
    it('should return undefined if time is invalid', () => {
      const timeConverter: TimeConverter = new TimeConverter();
      expect(
        timeConverter.utcUnixEpochDayFromTime('28:00', 'Europe/Paris'),
      ).toBeUndefined();
    });
    it('should return undefined if time is undefined', () => {
      const timeConverter: TimeConverter = new TimeConverter();
      expect(
        timeConverter.utcUnixEpochDayFromTime(undefined, 'Europe/Paris'),
      ).toBeUndefined();
    });
    it('should return undefined if timezone is invalid', () => {
      const timeConverter: TimeConverter = new TimeConverter();
      expect(
        timeConverter.utcUnixEpochDayFromTime('12:00', 'Foo/Bar'),
      ).toBeUndefined();
    });
    it('should return undefined if timezone is undefined', () => {
      const timeConverter: TimeConverter = new TimeConverter();
      expect(
        timeConverter.utcUnixEpochDayFromTime('12:00', undefined),
      ).toBeUndefined();
    });
  });

  describe('localUnixEpochDayFromTime', () => {
    it('should get the day of paris at 12:00 utc', () => {
      const timeConverter: TimeConverter = new TimeConverter();
      expect(
        timeConverter.localUnixEpochDayFromTime('12:00', 'Europe/Paris'),
      ).toBe(4);
    });
    it('should get the day of paris at 23:00 utc', () => {
      const timeConverter: TimeConverter = new TimeConverter();
      expect(
        timeConverter.localUnixEpochDayFromTime('23:00', 'Europe/Paris'),
      ).toBe(5);
    });
    it('should get the day of papeete at 05:00 utc', () => {
      const timeConverter: TimeConverter = new TimeConverter();
      expect(
        timeConverter.localUnixEpochDayFromTime('05:00', 'Pacific/Tahiti'),
      ).toBe(3);
    });
    it('should return undefined if time is invalid', () => {
      const timeConverter: TimeConverter = new TimeConverter();
      expect(
        timeConverter.localUnixEpochDayFromTime('28:00', 'Europe/Paris'),
      ).toBeUndefined();
    });
    it('should return undefined if time is undefined', () => {
      const timeConverter: TimeConverter = new TimeConverter();
      expect(
        timeConverter.localUnixEpochDayFromTime(undefined, 'Europe/Paris'),
      ).toBeUndefined();
    });
    it('should return undefined if timezone is invalid', () => {
      const timeConverter: TimeConverter = new TimeConverter();
      expect(
        timeConverter.localUnixEpochDayFromTime('12:00', 'Foo/Bar'),
      ).toBeUndefined();
    });
    it('should return undefined if timezone is undefined', () => {
      const timeConverter: TimeConverter = new TimeConverter();
      expect(
        timeConverter.localUnixEpochDayFromTime('12:00', undefined),
      ).toBeUndefined();
    });
  });
});
